// Copyright 2020 TiKV Project Authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package replication

import "github.com/prometheus/client_golang/prometheus"

var (
	drTickCounter = prometheus.NewCounter(
		prometheus.CounterOpts{
			Namespace: "pd",
			Subsystem: "replication",
			Name:      "dr_tick_total",
			Help:      "Counter of background state check count",
		})

	drStateGauge = prometheus.NewGauge(
		prometheus.GaugeOpts{
			Namespace: "pd",
			Subsystem: "replication",
			Name:      "dr_state",
			Help:      "State of DR",
		})

	drStateIDGauge = prometheus.NewGauge(
		prometheus.GaugeOpts{
			Namespace: "pd",
			Subsystem: "replication",
			Name:      "dr_state_id",
			Help:      "State ID of DR",
		})

	drRecoveredRegionGauge = prometheus.NewGauge(
		prometheus.GaugeOpts{
			Namespace: "pd",
			Subsystem: "replication",
			Name:      "dr_recovered_region",
			Help:      "Number of recovered regions",
		})

	drRecoverProgressGauge = prometheus.NewGauge(
		prometheus.GaugeOpts{
			Namespace: "pd",
			Subsystem: "replication",
			Name:      "dr_recover_progress",
			Help:      "Progress of sync_recover process",
		})
)

func init() {
	prometheus.MustRegister(drTickCounter)
	prometheus.MustRegister(drStateGauge)
	prometheus.MustRegister(drStateIDGauge)
	prometheus.MustRegister(drRecoveredRegionGauge)
	prometheus.MustRegister(drRecoverProgressGauge)
}
